#include "basic_objects.h"

#include "Creature.h"
#include <stdio.h>


// BLOODY MESS

BloodyMess::BloodyMess()
:	Object( "a bloody mess" )
{
}

BloodyMess::~BloodyMess()
{
}

BloodyMessInfo::BloodyMessInfo()
{
	bmp[0].file = "/add-ons/gfx/bloody_mess/bloody_mess";
	bmp[0].id = 'BlMe';

	bmp[1].file = NULL;
	bitmaps = bmp;
	name = "bloody_mess";
}

Object *
BloodyMessInfo::Create()
{
	BloodyMess * mess = new BloodyMess;
	strcpy(mess->bitmap, "add-ons/gfx/bloody_mess/bloody_mess");

	mess->internal_name = name;
	
	return mess;
}

bool
BloodyMessInfo::ShouldPlace( int level )
{
	return false;
}

// HEALTH POTION

HealthPotion::HealthPotion()
:	Object("a health potion")
{
	used = false;
}

HealthPotion::~HealthPotion()
{
}

void
HealthPotion::Use( Creature * target, Creature * )
{
	if (used)
		return;
	
	int gain = random() % 10+5;
	target->info.HP.curr += gain;
	if (target->info.HP.curr > target->info.HP.base)
		target->info.HP.curr = target->info.HP.base;
	
	char text[256];
	sprintf(text, "*clunk* *clunk* - %s gains %d hitpoints", target->name, gain );
	game_message( target->info.pos, text, TextMode::BOLD, (rgb_color){128,128,255,255} );
	
	used = true;
	strcpy(bitmap, "add-ons/gfx/empty_potion/empty_potion");
}

status_t
HealthPotion::Save( BFile * file )
{
	if ( file->Write( &used, sizeof(used)) != sizeof(used) )
		return B_ERROR;
	
	return Object::Save(file);
}

status_t
HealthPotion::Load( BFile * file )
{
	if ( file->Read( &empty_id, sizeof(empty_id)) != sizeof(empty_id) )
		return B_ERROR;
		
	if ( file->Read( &used, sizeof(used)) != sizeof(used) )
		return B_ERROR;
	
	return Object::Load(file);
}

HealthPotionInfo::HealthPotionInfo()
{
	bmp[0].file = "/add-ons/gfx/blue_potion/blue_potion";
	bmp[0].id = 'BlBo';
	bmp[1].file = "/add-ons/gfx/empty_potion/empty_potion";
	bmp[1].id = 'EmBo';
	bmp[2].file = NULL;
	bitmaps = bmp;

	name = "potion_health";
}

Object *
HealthPotionInfo::Create()
{
	HealthPotion * potion = new HealthPotion;
	strcpy(potion->bitmap, "add-ons/gfx/blue_potion/blue_potion");

	potion->internal_name = name;
	
	return potion;
}

// Sword

Sword::Sword()
:	Object("a sword")
{
	skill = "1H Sword"; // uses skill '1H Sword'
	equip = Creature::RIGHT_HAND; // only equipable in right hand (weapon hand)
}

Sword::~Sword()
{
}

void
Sword::Use( Creature * target, Creature * source )
{
	if (!source)
	{
		game_message( Pos(0,0,0), "Invalid source argument to Sword::Use()" );
		return;
	}
	
	damage dmg;
	dmg.amount = random() % 5 + source->info.STR.curr;
	dmg.source = source;
	dmg.type = damage::SLASH;
	
	char text[512];
	sprintf(text,"%s slashes %s for %d dmg",source->name,target->name,dmg.amount);
	game_message( Pos(0,0,0), text, TextMode::BOLD, (rgb_color){255,0,0,255} );
	target->Damage( dmg );
}

SwordInfo::SwordInfo()
{
	bmp[0].file = "/add-ons/gfx/sword/sword";
	bmp[0].id = 'SwBa';
	bmp[1].file = NULL;
	bitmaps = bmp;

	name = "sword_basic";
}

Object *
SwordInfo::Create()
{
	Sword * sword = new Sword;
	strcpy(sword->bitmap, "add-ons/gfx/sword/sword");

	sword->internal_name = name;
	
	return sword;
}

bool
SwordInfo::ShouldPlace( int level )
{
	if ( (random() % 5) == 0 )
		return true; // only place a sword 20% of given opportunities
	return false;
}

// BUILDER

BasicObjectBuilder::BasicObjectBuilder()
:	ObjectBuilder( "BasicObjectBuilder" )
{
	info_original[0] = &health_info;
	info_original[1] = &mess_info;
	info_original[2] = &sword_info;
	info_original[3] = NULL;
	info = info_original;

	text = "BasicObjectBuilder";
}

// EXPORT

ObjectBuilder * 
CreateObjectBuilder()
{
	return new BasicObjectBuilder;
}
