#ifndef LNLIST_H
#define LNLIST_H

#include <stdlib.h>
#include <memory.h>
#include <StopWatch.h>

/* Dynamic array, by Mikael Eiman 1999 */

template <class T>
class List
{
	protected:
		T ** array;
		long num_items;
		long array_size;
		bool delete_items;
		void SubSort(long, long); // used by sort
		
	public:
		List();
		~List();
		
		void operator += (T*);        // add an item
		void operator -= (T*);        // remove an item
		T * operator [] (long) const;   // access an item
		long NumItems();

		void Sort();
		void SetDeleteItems(bool d) { delete_items = d; };
		
		void Clear( bool );
};

template <class T>
List<T>::List()
:	num_items(0),
	array_size(4),
	delete_items(false)
{
	array = new T*[array_size];
}

template <class T>
List<T>::~List()
{
	if (delete_items)
	{
		for (long c=0;(*this)[c];c++)
			delete (*this)[c];
	}
	
	delete [] array;
}

template <class T>
void List<T>::operator += (T * item)
{
	if (num_items == array_size)
	{ // alloc new, bigger array
		array_size *= 2;
		T ** old = array;
		array = new T*[array_size];
		memset(array,0,sizeof(T*)*array_size);
		memcpy(array,old,sizeof(T*) * num_items);
		delete [] old;
	}
	array[num_items] = item;
	num_items++;
}

template <class T>
void List<T>::operator -= (T * item)
{
	for (long c=0;c<num_items;c++)
	{
		if (array[c] == item)
		{
			if (c < num_items-1)
			{
				memmove(&array[c],&array[c+1],sizeof(T*)*(num_items-c-1));
			}
			num_items--;
			return; // only remove ONE instance
		}
	}
}

template <class T>
T * List<T>::operator [] (long i) const
{
	if (i < num_items && i >= 0)
		return array[i];
	else
		return NULL;
}

template <class T>
long List<T>::NumItems()
{
	return num_items;
}

template <class T>
void List<T>::Sort()
{
	BStopWatch timer("LnList::Sort");
	if (num_items < 2)
		return; // no point in sorting a single item, is there?

	// sort array
	SubSort(0,num_items-1);
}

template <class T>
void List<T>::SubSort(long a, long b)
{ // stolen from one of the JDK demos
	if (a < b) 
	{ // sanity check
		T * divider = array[(a+b)/2];
		long lo = a, hi = b;
		while (lo <= hi) 
		{
			while ( (lo < b) && (*array[lo] < *divider) ) ++lo;
			while ( (hi > a) && (*array[hi] > *divider) ) --hi;
			if (lo <= hi) 
			{ // swap
				T * temp = array[lo]; 
				array[lo] = array[hi]; 
				array[hi] = temp;
				++lo; --hi;
			}
		}
		if (a < hi)
      	SubSort(a,hi);
		if (lo < b)
      	SubSort(lo,b);
	}
}

template <class T>
void List<T>::Clear( bool _delete )
{
	if (_delete)
	{
		for (long c=0;(*this)[c];c++)
			delete (*this)[c];
	}
	
	delete [] array;
	
	num_items = 0;
	array_size = 4;
	array = new T*[array_size];
}

#endif

