#ifndef MAIN_WINDOW_H
#define MAIN_WINDOW_H

#include <Window.h>
#include <MenuBar.h>

#include "Dungeon.h"
#include "PlayerView.h"
#include "InfoView.h"
#include "MapView.h"

class DungeonView : public BView
{
	public:
		DungeonView( BRect, MapView * );
		virtual ~DungeonView();

		virtual void Draw( BRect );

		virtual void MessageReceived( BMessage * );
		virtual void AttachedToWindow();
		
		void ScrollToPlayer();
		void ModifyLight( float );
		void UpdateMap();
		
	protected:
		MapView	* mapView;
		
		int off_x, off_y;
		int zoom;
		
		int Width(); // in dungeon squares
		int Height(); // in dungeon squares
		
		void PlayerMove( direction );
		
		void ScrollTo( int, int ); // move offset to x,y and redraw view

		void Scroll( direction );
		void Zoom( int );
		
		void ClimbStairs();
		
		void Animation( const char *, Pos );
};

class MainWindow : public BWindow
{
	public:
		enum {
			TOGGLE_MAP,	// toggle map display on/off
			CLIMB_UP,
			CLIMB_DOWN
		};

		MainWindow( BRect );
		virtual ~MainWindow();

		virtual bool QuitRequested();
		virtual void MessageReceived( BMessage * );
		virtual void DispatchMessage( BMessage * msg, BHandler * handler );
		virtual void Zoom( BPoint, float, float );
				
		void Generate( Creature *, int, int );
		void SetCurrentDungeon ( Dungeon *, bool heading_up );
		void SetCurrentDungeon( int );
		
		void Setup( Graphics*, CreatureInfoSet*,ObjectInfoSet*);
		
		void Save();
		void Load();
		
	protected:
		Dungeon			* dungeons;
		
		DungeonView		* dungeonView;
		PlayerView		* playerView;
		InfoView		* infoView;
		MapView			* mapView;
		BView			* groundView;
		
		int32			num_levels, curr_level;
		Graphics		* graphics;
		CreatureInfoSet	* creature_info;
		ObjectInfoSet	* object_info;
		
		CreatureSet		creatures;
		ObjectSet		objects;
};

#endif
